@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: totemNarrowSpacePlugin.cpp 3413 2006-08-11 14:15:44Z hadess $
+ * $Id: totemNarrowSpacePlugin.cpp,v 1.5 2006/10/01 12:05:42 hadess Exp $
  */
 
 #include <mozilla-config.h>
@@ -44,6 +44,9 @@ static const char kPluginDescription[] = "QuickTime Plug-in 7.0 (compatible; Tot
 
 static const totemPluginMimeEntry kMimeTypes[] = {
 	{ "video/quicktime", "mov", NULL },
+	{ "video/mp4", "mp4", NULL },
+	{ "image/x-macpaint", "pntg", NULL },
+	{ "image/x-quicktime", "pict, pict1, pict2", "image/x-pict" }
 };
 
 totemScriptablePlugin::totemScriptablePlugin (totemPlugin *aPlugin)
