@@ -50,8 +50,9 @@ typedef struct {
 	guint hidden : 1;
 	guint repeat : 1;
 	guint is_playlist : 1;
+	guint tried_write : 1;
 	guint noautostart : 1;
-	//guint srcSupported : 1;
+	guint is_supported_src : 1;
 } totemPlugin;
 
 typedef struct {
