@@ -204,6 +204,8 @@ typedef enum {
 	BVW_INFO_HAS_AUDIO,
 	BVW_INFO_AUDIO_BITRATE,
 	BVW_INFO_AUDIO_CODEC,
+	BVW_INFO_AUDIO_SAMPLE_RATE,
+	BVW_INFO_AUDIO_CHANNELS
 } BaconVideoWidgetMetadataType;
 
 void bacon_video_widget_get_metadata		 (BaconVideoWidget *bvw,
@@ -304,6 +306,9 @@ int bacon_video_widget_get_subtitle              (BaconVideoWidget *bvw);
 void bacon_video_widget_set_subtitle             (BaconVideoWidget *bvw,
 		                                  int subtitle);
 
+gboolean bacon_video_widget_has_next_track	 (BaconVideoWidget *bvw);
+gboolean bacon_video_widget_has_previous_track	 (BaconVideoWidget *bvw);
+
 /* Screenshot functions */
 gboolean bacon_video_widget_can_get_frames       (BaconVideoWidget *bvw,
 						  GError **error);
