@@ -70,6 +70,9 @@ struct TotemPlaylistPrivate
 	/* This is the playing icon */
 	GdkPixbuf *icon;
 
+	GtkActionGroup *action_group;
+	GtkUIManager *ui_manager;
+
 	/* This is a scratch list for when we're removing files */
 	GList *list;
 
@@ -131,6 +134,14 @@ static const GtkTargetEntry target_table[] = {
 	{ "_NETSCAPE_URL", 0, 1 },
 };
 
+static void playlist_remove_action_activated (GtkAction *action, TotemPlaylist *playlist);
+static void playlist_copy_location_action_activated (GtkAction *action, TotemPlaylist *playlist);
+
+static const GtkActionEntry entries[] = {
+	{ "remove", GTK_STOCK_REMOVE, N_("_Remove"), NULL, N_("Remove file from playlist"), G_CALLBACK (playlist_remove_action_activated) },
+	{ "copy-location", GTK_STOCK_COPY, N_("_Copy location"), NULL, N_("Copy the location to the clipboard"), G_CALLBACK (playlist_copy_location_action_activated) }
+};
+
 static GtkWidgetClass *parent_class = NULL;
 
 static void totem_playlist_class_init (TotemPlaylistClass *class);
@@ -452,7 +463,7 @@ drop_cb (GtkWidget     *widget,
 }
 
 static void
-on_copy1_activate (GtkButton *button, TotemPlaylist *playlist)
+playlist_copy_location_action_activated (GtkAction *action, TotemPlaylist *playlist)
 {
 	GList *l;
 	GtkTreePath *path;
@@ -485,35 +496,69 @@ on_copy1_activate (GtkButton *button, TotemPlaylist *playlist)
 }
 
 static gboolean
-treeview_button_pressed (GtkTreeView *treeview, GdkEventButton *event,
-		TotemPlaylist *playlist)
+playlist_show_popup_menu (TotemPlaylist *playlist, GdkEventButton *event)
 {
+	guint button = 0;
+       	guint32 time;
 	GtkTreePath *path;
+	gint count;
 	GtkWidget *menu;
+	GtkAction *action; 
 
-	if (event->type != GDK_BUTTON_PRESS
-			|| event->button != 3)
-		return FALSE;
+	if (event != NULL) {
+		button = event->button;
+		time = event->time;
 
-	if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (treeview),
-				event->x, event->y, &path,
-				NULL, NULL, NULL) == FALSE)
-	{
+		if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (playlist->_priv->treeview),
+				 event->x, event->y, &path, NULL, NULL, NULL)) {
+			if (!gtk_tree_selection_path_is_selected (playlist->_priv->selection, path)) {
+				gtk_tree_selection_unselect_all (playlist->_priv->selection);
+				gtk_tree_selection_select_path (playlist->_priv->selection, path);
+			}
+			gtk_tree_path_free (path);
+		} else {
+			gtk_tree_selection_unselect_all (playlist->_priv->selection);
+		}
+	} else {
+		time = gtk_get_current_event_time ();
+	}
+
+	count = gtk_tree_selection_count_selected_rows (playlist->_priv->selection);
+	
+	if (count == 0) {
 		return FALSE;
 	}
 
-	gtk_tree_selection_unselect_all (playlist->_priv->selection);
-	gtk_tree_selection_select_path (playlist->_priv->selection, path);
-	gtk_tree_path_free (path);
+	action = gtk_action_group_get_action (playlist->_priv->action_group, "copy-location");
+	gtk_action_set_sensitive (action, count == 1);
+
+	menu = gtk_ui_manager_get_widget (playlist->_priv->ui_manager, "/totem-playlist-popup");
 
-	menu = glade_xml_get_widget (playlist->_priv->xml, "menu1");
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
-			event->button, event->time);
 	gtk_menu_shell_select_first (GTK_MENU_SHELL (menu), FALSE);
 
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
+			button, time);
+
 	return TRUE;
 }
 
+static gboolean
+treeview_button_pressed (GtkTreeView *treeview, GdkEventButton *event,
+		TotemPlaylist *playlist)
+{
+	if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
+		return playlist_show_popup_menu (playlist, event);
+	}
+
+	return FALSE;
+}
+
+static gboolean
+playlist_treeview_popup_menu (GtkTreeView *treeview, TotemPlaylist *playlist)
+{
+	return playlist_show_popup_menu (playlist, NULL);
+}
+
 static void
 totem_playlist_set_reorderable (TotemPlaylist *playlist, gboolean set)
 {
@@ -718,7 +763,7 @@ totem_playlist_foreach_selected (GtkTreeModel *model, GtkTreePath *path,
 }
 
 static void
-totem_playlist_remove_files (GtkWidget *widget, TotemPlaylist *playlist)
+playlist_remove_files (TotemPlaylist *playlist)
 {
 	GtkTreeSelection *selection;
 	GtkTreeRowReference *ref;
@@ -825,6 +870,18 @@ totem_playlist_remove_files (GtkWidget *widget, TotemPlaylist *playlist)
 	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (playlist->_priv->treeview));
 }
 
+static void
+playlist_remove_button_clicked (GtkWidget *button, TotemPlaylist *playlist)
+{
+	playlist_remove_files (playlist);
+}
+
+static void
+playlist_remove_action_activated (GtkAction *action, TotemPlaylist *playlist)
+{
+	playlist_remove_files (playlist);
+}
+
 static void
 totem_playlist_save_playlist (TotemPlaylist *playlist, char *filename, gint active_format)
 {
@@ -889,6 +946,7 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 			NULL);
 	gtk_dialog_set_default_response (GTK_DIALOG (fs), GTK_RESPONSE_ACCEPT);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (fs), FALSE);
+	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (fs), TRUE);
 	/* translators: Playlist is the default saved playlist filename,
 	 * without the suffix */
 	filename = g_strconcat (_("Playlist"), save_types[0].suffix, NULL);
@@ -910,8 +968,6 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 	if (response == GTK_RESPONSE_ACCEPT)
 	{
 		char *filename;
-		GnomeVFSResult res;
-		GnomeVFSFileInfo info;
 		gint active_format;
 
 		filename = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (fs));
@@ -925,35 +981,6 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 		g_free (playlist->_priv->save_path);
 		playlist->_priv->save_path = g_path_get_dirname (filename);
 
-		res = gnome_vfs_get_file_info (filename, &info, GNOME_VFS_FILE_INFO_DEFAULT);
-		if (res != GNOME_VFS_ERROR_NOT_FOUND && res == GNOME_VFS_OK)
-		{
-			GtkWidget *dialog;
-
-			dialog = gtk_message_dialog_new
-				(totem_playlist_get_toplevel (playlist),
-				 GTK_DIALOG_MODAL,
-				 GTK_MESSAGE_QUESTION,
-				 GTK_BUTTONS_NONE,
-				 _("Overwrite file?"));
-			gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-								  _("A file named '%s' already exists.  Are you sure you want to overwrite it?"),
-								  filename);
-
-			gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-			gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-					GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
-					GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-					NULL);
-
-			response = gtk_dialog_run (GTK_DIALOG (dialog));
-			gtk_widget_destroy (dialog);
-			if (response != GTK_RESPONSE_ACCEPT)
-			{
-				g_free (filename);
-				return;
-			}
-		}
 		totem_playlist_save_playlist (playlist, filename, active_format);
 		g_free (filename);
 	} else {
@@ -1089,33 +1116,6 @@ totem_playlist_down_files (GtkWidget *widget, TotemPlaylist *playlist)
 	totem_playlist_move_files (playlist, FALSE);
 }
 
-static void
-totem_playlist_popup_menu (TotemPlaylist *playlist, guint time)
-{
-	GtkTreePath *path;
-	GtkWidget *menu;
-	int x, y;
-
-	if (gdk_window_get_pointer (playlist->_priv->treeview->window,
-			&x, &y, NULL) == NULL)
-		return;
-
-	if (gtk_tree_view_get_path_at_pos
-			(GTK_TREE_VIEW (playlist->_priv->treeview),
-			 x, y, &path, NULL, NULL, NULL) == FALSE) {
-		return;
-	}
-
-	gtk_tree_selection_unselect_all (playlist->_priv->selection);
-	gtk_tree_selection_select_path (playlist->_priv->selection, path);
-	gtk_tree_path_free (path);
-
-	menu = glade_xml_get_widget (playlist->_priv->xml, "menu1");
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
-			0, time);
-	gtk_menu_shell_select_first (GTK_MENU_SHELL (menu), FALSE);
-}
-
 static int
 totem_playlist_key_press (GtkWidget *win, GdkEventKey *event, TotemPlaylist *playlist)
 {
@@ -1127,17 +1127,6 @@ totem_playlist_key_press (GtkWidget *win, GdkEventKey *event, TotemPlaylist *pla
 				(playlist->_priv->selection);
 			return TRUE;
 		}
-
-		if (event->state & GDK_SHIFT_MASK
-		    && event->keyval == GDK_F10) {
-			totem_playlist_popup_menu (playlist, event->time);
-			return TRUE;
-		}
-	}
-
-	if (event->keyval == GDK_Menu) {
-		totem_playlist_popup_menu (playlist, event->time);
-		return TRUE;
 	}
 
 	/* If we have modifiers, and either Ctrl, Mod1 (Alt), or any
@@ -1153,7 +1142,7 @@ totem_playlist_key_press (GtkWidget *win, GdkEventKey *event, TotemPlaylist *pla
 
 	if (event->keyval == GDK_Delete)
 	{
-		totem_playlist_remove_files (NULL, playlist);
+		playlist_remove_files (playlist);
 		return TRUE;
 	}
 
@@ -1258,6 +1247,8 @@ init_treeview (GtkWidget *treeview, TotemPlaylist *playlist)
 			G_CALLBACK (treeview_row_changed), playlist);
 	g_signal_connect (G_OBJECT (treeview), "button-press-event",
 			G_CALLBACK (treeview_button_pressed), playlist);
+	g_signal_connect (G_OBJECT (treeview), "popup-menu",
+			G_CALLBACK (playlist_treeview_popup_menu), playlist);
 
 	/* Drag'n'Drop */
 	g_signal_connect (G_OBJECT (treeview), "drag_data_received",
@@ -1467,6 +1458,14 @@ totem_playlist_finalize (GObject *object)
 		gdk_pixbuf_unref (playlist->_priv->icon);
 	g_object_unref (playlist->_priv->parser);
 
+	if (playlist->_priv->ui_manager != NULL) {
+		g_object_unref (G_OBJECT (playlist->_priv->ui_manager));
+	}
+
+	if (playlist->_priv->action_group != NULL) {
+		g_object_unref (G_OBJECT (playlist->_priv->action_group));
+	}
+
 	if (G_OBJECT_CLASS (parent_class)->finalize != NULL) {
 		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 	}
@@ -1477,6 +1476,7 @@ totem_playlist_new (void)
 {
 	TotemPlaylist *playlist;
 	GtkWidget *container, *item;
+	char *filename;
 
 	playlist = TOTEM_PLAYLIST (g_object_new (GTK_TYPE_PLAYLIST, NULL));
 
@@ -1489,6 +1489,25 @@ totem_playlist_new (void)
 		return NULL;
 	}
 
+	/* popup menu */
+	playlist->_priv->action_group = 
+		gtk_action_group_new ("playlist-action-group");
+	gtk_action_group_set_translation_domain (playlist->_priv->action_group,
+			GETTEXT_PACKAGE);
+	gtk_action_group_add_actions (playlist->_priv->action_group, entries,
+			G_N_ELEMENTS (entries), playlist);
+
+	playlist->_priv->ui_manager = gtk_ui_manager_new ();
+	gtk_ui_manager_insert_action_group (playlist->_priv->ui_manager,
+			playlist->_priv->action_group, -1);
+
+	filename = totem_interface_get_full_path ("playlist-ui.xml");
+	if (!gtk_ui_manager_add_ui_from_file (playlist->_priv->ui_manager, filename, NULL)) {
+		totem_playlist_finalize (G_OBJECT (playlist));
+		return NULL;
+	}
+	g_free (filename);
+
 	/* Connect the buttons */
 	item = glade_xml_get_widget (playlist->_priv->xml, "add_button");
 	g_signal_connect (G_OBJECT (item), "clicked",
@@ -1496,7 +1515,7 @@ totem_playlist_new (void)
 			playlist);
 	item = glade_xml_get_widget (playlist->_priv->xml, "remove_button");
 	g_signal_connect (G_OBJECT (item), "clicked",
-			G_CALLBACK (totem_playlist_remove_files),
+			G_CALLBACK (playlist_remove_button_clicked),
 			playlist);
 	item = glade_xml_get_widget (playlist->_priv->xml, "save_button");
 	g_signal_connect (G_OBJECT (item), "clicked",
@@ -1511,10 +1530,6 @@ totem_playlist_new (void)
 			G_CALLBACK (totem_playlist_down_files),
 			playlist);
 
-	item = glade_xml_get_widget (playlist->_priv->xml, "copy1");
-	g_signal_connect (G_OBJECT (item), "activate",
-			G_CALLBACK (on_copy1_activate), playlist);
-
 	gtk_widget_add_events (GTK_WIDGET (playlist), GDK_KEY_PRESS_MASK);
 	g_signal_connect (G_OBJECT (playlist), "key_press_event",
 			G_CALLBACK (totem_playlist_key_press), playlist);
@@ -1900,7 +1915,7 @@ totem_playlist_has_next_mrl (TotemPlaylist *playlist)
 }
 
 gboolean
-totem_playlist_set_title (TotemPlaylist *playlist, const char *title)
+totem_playlist_set_title (TotemPlaylist *playlist, const char *title, gboolean force)
 {
 	GtkListStore *store;
 	GtkTreeIter iter;
@@ -1919,11 +1934,13 @@ totem_playlist_set_title (TotemPlaylist *playlist, const char *title)
 	if (&iter == NULL)
 		return FALSE;
 
-	gtk_tree_model_get (playlist->_priv->model, &iter,
-			TITLE_CUSTOM_COL, &custom_title,
-			-1);
-	if (custom_title != FALSE)
-		return TRUE;
+	if (force == FALSE) {
+		gtk_tree_model_get (playlist->_priv->model, &iter,
+				TITLE_CUSTOM_COL, &custom_title,
+				-1);
+		if (custom_title != FALSE)
+			return TRUE;
+	}
 
 	gtk_list_store_set (store, &iter,
 			FILENAME_COL, title,
