@@ -18,6 +18,14 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
+ * The Totem project hereby grant permission for non-gpl compatible GStreamer
+ * plugins to be used and distributed together with GStreamer and Totem. This
+ * permission are above and beyond the permissions granted by the GPL license
+ * Totem is covered by.
+ *
+ * Monday 7th February 2005: Christian Schaller: Add excemption clause.
+ * See license_change file for details.
+ *
  */
 
 #include "config.h"
@@ -57,7 +65,7 @@ totem_get_icon_from_theme (const char *id, GtkIconSize size)
 	return pixbuf;
 }
 
-static gboolean 
+static gboolean
 remove_value (gpointer key, GdkPixbuf *pixbuf, Totem *totem)
 {
 	gdk_pixbuf_unref (pixbuf);
@@ -86,63 +94,48 @@ totem_default_theme_changed (GtkIconTheme *theme, Totem *totem)
 	totem_set_default_icons (totem);
 }
 
+static void
+set_icon_for_action (Totem *totem, const char *name, const char *icon_id)
+{
+	GtkAction *action;
+	GSList *proxies, *p;
+
+	action = gtk_action_group_get_action (totem->main_action_group, name);
+
+	g_object_set_data (G_OBJECT (action), "pixbuf-icon",
+			PIXBUF_FOR_ID(icon_id));
+	proxies = g_slist_copy (gtk_action_get_proxies (action));
+	for (p = proxies; p != NULL; p = p->next) {
+		gtk_action_connect_proxy (action, GTK_WIDGET (p->data));
+	}
+	g_slist_free (proxies);
+}
+
 void
 totem_set_default_icons (Totem *totem)
 {
 	GtkWidget *item;
 
-	/* Play button */
-	item = glade_xml_get_widget (totem->xml,
-			"tmw_play_pause_button_image");
-	gtk_image_set_from_pixbuf (GTK_IMAGE (item),
-			totem->state == STATE_PLAYING
-			? PIXBUF_FOR_ID("stock-media-pause")
-			: PIXBUF_FOR_ID("stock-media-play"));
-	item = glade_xml_get_widget (totem->xml, "tcw_pp_button_image");
-	gtk_image_set_from_pixbuf (GTK_IMAGE (item),
-			totem->state == STATE_PLAYING
-			? PIXBUF_FOR_ID("stock-media-pause")
-			: PIXBUF_FOR_ID("stock-media-play"));
-
-	/* Previous button */
-	item = glade_xml_get_widget (totem->xml, "tmw_previous_button_image");
-	gtk_image_set_from_pixbuf (GTK_IMAGE (item),
-			PIXBUF_FOR_ID("stock-media-prev"));
-	item = glade_xml_get_widget (totem->xml, "tcw_previous_button_image");
-	gtk_image_set_from_pixbuf (GTK_IMAGE (item),
-			PIXBUF_FOR_ID("stock-media-prev"));
-
-	/* Next button */
-	item = glade_xml_get_widget (totem->xml, "tmw_next_button_image");
-	gtk_image_set_from_pixbuf (GTK_IMAGE (item),
-			PIXBUF_FOR_ID("stock-media-next"));
-	item = glade_xml_get_widget (totem->xml, "tcw_next_button_image");
-	gtk_image_set_from_pixbuf (GTK_IMAGE (item),
-			PIXBUF_FOR_ID("stock-media-next"));
-
 	/* Screenshot button */
-	item = glade_xml_get_widget (totem->xml,
-			"tmw_take_screenshot_menu_item_image");
-	gtk_image_set_from_pixbuf (GTK_IMAGE (item),
-			PIXBUF_FOR_ID("panel-screenshot"));
+	set_icon_for_action (totem, "take-screenshot", "panel-screenshot");
+	set_icon_for_action (totem, "volume-up", "audio-volume-high");
+	set_icon_for_action (totem, "volume-down", "audio-volume-low");
 
-	/* Playlist button */
+	/* Leave fullscreen button */
 	item = glade_xml_get_widget (totem->xml,
-			"tmw_playlist_button_image");
+			"tefw_image");
 	gtk_image_set_from_pixbuf (GTK_IMAGE (item),
-			PIXBUF_FOR_ID("stock_playlist"));
-	item = glade_xml_get_widget (totem->xml,
-			"tmw_show_playlist_menu_item_image");
-	gtk_image_set_from_pixbuf (GTK_IMAGE (item),
-			PIXBUF_FOR_ID("stock_playlist"));
-	gtk_window_set_icon (GTK_WINDOW (totem->playlist),
-			PIXBUF_FOR_ID("stock_playlist"));
-}
+			PIXBUF_FOR_ID("stock_leave-fullscreen"));
 
-GdkPixbuf *
-totem_get_named_icon_for_id (const char *id)
-{
-	return PIXBUF_FOR_ID(id);
+	/* Fullscreen audio-volume buttons */
+	item = glade_xml_get_widget (totem->xml,
+			"tcw_volume_mute_image");
+	gtk_image_set_from_pixbuf (GTK_IMAGE (item),
+			PIXBUF_FOR_ID("audio-volume-mute"));
+	item = glade_xml_get_widget (totem->xml,
+			"tcw_volume_max_image");
+	gtk_image_set_from_pixbuf (GTK_IMAGE (item),
+			PIXBUF_FOR_ID("audio-volume-high"));
 }
 
 static GdkPixbuf *
@@ -165,18 +158,12 @@ totem_named_icons_init (Totem *totem, gboolean refresh)
 {
 	GtkIconTheme *theme;
 	int i;
-
-	struct {
-		const char *fn1;
-		const char *fn2;
-		const char *fn3;
-	} items[] = {
-		{ "stock-media-play", "stock_media-play", "stock_media_play" },
-		{ "stock-media-pause", "stock_media-pause", "stock_media_pause" },
-		{ "stock-media-prev", "stock_media-prev", "stock_media_previous" },
-		{ "stock-media-next", "stock_media-next", "stock_media_next" },
-		{ "panel-screenshot", "stock-panel-screenshot", "gnome-screenshot" },
-		{ "stock_playlist", "playlist-24", NULL },
+	char *items[][4] = {
+		{ "panel-screenshot", "stock-panel-screenshot", "gnome-screenshot", "applets-screenshooter" },
+		{ "stock_leave-fullscreen", GTK_STOCK_QUIT, NULL, NULL },
+		{ "audio-volume-high", "stock_volume-max", NULL, NULL },
+		{ "audio-volume-low", "stock_volume-min", NULL, NULL },
+		{ "audio-volume-mute", "stock_volume-mute", NULL, NULL },
 	};
 
 	if (refresh == FALSE) {
@@ -192,53 +179,49 @@ totem_named_icons_init (Totem *totem, gboolean refresh)
 
 	for (i = 0; i < (int) G_N_ELEMENTS (items); i++) {
 		GdkPixbuf *pixbuf = NULL;
+		int size;
+		int j;
 
-		pixbuf = totem_get_icon_from_theme (items[i].fn1,
-				GTK_ICON_SIZE_BUTTON);
+		size = G_N_ELEMENTS (items[i]);
 
-		if (pixbuf == NULL && items[i].fn2 != NULL) {
-			pixbuf = totem_get_icon_from_theme (items[i].fn2,
-					GTK_ICON_SIZE_BUTTON);
-		}
-
-		if (pixbuf == NULL && items[i].fn3 != NULL) {
-			pixbuf = totem_get_icon_from_theme (items[i].fn3,
-					GTK_ICON_SIZE_BUTTON);
-		}
-
-		if (pixbuf == NULL && items[i].fn3 != NULL) {
-			pixbuf = totem_get_pixbuf_from_totem_install
-				(items[i].fn3);
+		for (j = 0; j < size && items[i][j] != NULL; j++) {
+			pixbuf = totem_get_icon_from_theme
+				(items[i][j],
+				 GTK_ICON_SIZE_BUTTON);
+			if (pixbuf != NULL)
+				break;
 		}
 
 		if (pixbuf == NULL) {
-			pixbuf = totem_get_pixbuf_from_totem_install
-				(items[i].fn2);
+			for (j = 0; j < size && items[i][j] != NULL; j++) {
+				pixbuf = totem_get_pixbuf_from_totem_install
+					(items[i][j]);
+				if (pixbuf != NULL)
+					break;
+			}
 		}
 
 		if (pixbuf == NULL) {
-			pixbuf = totem_get_pixbuf_from_totem_install
-				(items[i].fn1);
+			for (j = 0; j < size && items[i][j] != NULL; j++) {
+				pixbuf = gtk_widget_render_icon
+					(GTK_WIDGET (totem->win),
+					 items[i][j], GTK_ICON_SIZE_BUTTON,
+					 NULL);
+				if (pixbuf != NULL)
+					break;
+			}
 		}
 
 		if (pixbuf == NULL) {
 			g_warning ("Couldn't find themed icon for \"%s\"",
-					items[i].fn1);
+					items[i][0]);
 			continue;
 		}
 
 		/* The pixbuf will be unref'ed when we destroy the 
 		 * hash table */
-		g_hash_table_insert (icons_table, (gpointer) items[i].fn1,
+		g_hash_table_insert (icons_table, (gpointer) items[i][0],
 				(gpointer) pixbuf);
 	}
-
-	/* Switch direction of the Play icon if we're in Right-To-Left */
-	if (gtk_widget_get_direction (totem->win) == GTK_TEXT_DIR_RTL)
-	{
-		totem_pixbuf_mirror (PIXBUF_FOR_ID("stock-media-play"));
-		totem_pixbuf_mirror (PIXBUF_FOR_ID("stock-media-next"));
-		totem_pixbuf_mirror (PIXBUF_FOR_ID("stock-media-prev"));
-	}
 }
 
